(* tag: category associated to a poset *)

(** * Category associated to a poset *)

Implicit Arguments On.

Require Export poset_.
Require Export cat_.
Require functor_.
Require misc_.

Module Cat_Of_Poset_.
Export Poset_.
Export Cat_.
Export Functor_.

Section Cat_Of_Poset.

Section Objects.

Variable A:Poset.

Local M [x,y:A] : Type := (leq x y).

Local ooM [x,y,z:A; f:(M y z); g:(M x y)] : (M x z).
Proof.
  Unfold M; EAuto.
Qed.

Local iiM [x:A] : (M x x).
Proof.
  Unfold M; Auto.
Qed.

Definition cat_of_poset : Cat.
Proof.
  Split with (Build_Cat_Data ooM iiM).
  Hints Resolve misc_.Misc_.proof_irrelevance.
  Abstract Split; Simpl; Unfold M; Auto.
Defined.  

End Objects.

Section Morphisms.

Variable A,B:Poset; f:(Poset_Morph A B).

Local h_obj [x:(cat_of_poset A)] : (cat_of_poset B) :=
 (f x).

Local h_mor [x,y:(cat_of_poset A); g:(mor x y)] :
  (mor (h_obj x) (h_obj y)).
Proof.
  Unfold h_obj cat_of_poset; Simpl; Intros.
  Abstract Auto.
Defined.

Remark h_fct : (Functor_Theory h_mor).
Proof.
  Hints Resolve misc_.Misc_.proof_irrelevance.
  Unfold h_mor h_obj cat_of_poset.
  Split; Simpl; Auto.
Qed.

Definition cat_of_poset_fct :
 (Functor (cat_of_poset A) (cat_of_poset B)).
Proof.
  Split with (Build_Functor_Data h_mor).
  Exact h_fct.
Defined.

End Morphisms.

End Cat_Of_Poset.

End Cat_Of_Poset_.
