(* tag: small_set has colimits *)

(** ** The category of sets has colimits. *)

Implicit Arguments On.

Require Export nt_.
Require misc_.
Require map_.

Module Colimit2_.

Export NT_.
Import Misc_.

Section Constant_Functor.

Variable A,B:Cat; u:B.

Local F : (Functor_Data A B).
Proof.
  Split with [x:A] u.
  Intros.
  Apply ii.
Defined.

Remark HF : (Functor_Theory (fct F)).
Proof.
  Unfold F; Split; Simpl; Intros.
  Rewrite ii_sx; Trivial.
  Reflexivity.
Qed.

Definition constant_fct : (Functor A B).
Proof.
  Split with F; Exact HF.
Defined.

End Constant_Functor.

Section Constant_NT.

Variable I,A:Cat; x,y:A; f:(mor x y).

Local eta [i:I] : (mor (constant_fct I x i) (constant_fct I y i)) := f.

Remark H_eta : (NT_Comm eta).
Proof.
  Unfold NT_Comm; Simpl; Intros.
  Rewrite ii_sx.
  Rewrite ii_dx.
  Reflexivity.
Qed.

Definition constant_nt : (NT (constant_fct I x) (constant_fct I y)).
Proof.
  Split with eta.
  Exact H_eta.
Defined.

End Constant_NT.

Section Colimit.

Variable I,A:Cat; D:(Functor I A).

Definition Cocone [x:A] : Type := (NT D (constant_fct I x)).

(* !!!! l'ipotesi che i cocone_diagram siano uguali si puo' togliere?
(Sono il dominio di trasformazioni naturali uguali.) *)
Lemma Cocone_extens : (x:A; cx1,cx2:(Cocone x))
 ((i:I)(eq_mor (cx1 i) (cx2 i))) -> cx1==cx2.
Proof.
  Unfold Cocone; Simpl; Intros.
  Apply nt_extens; Intros.
  Apply eq_mor_eqT.
  Apply H.
Qed.  

Section Cocone_Comp.

Variable x,y:A; f:(mor x y); cx:(Cocone x).

Local c [i:I] : (mor (D i) (constant_fct I y i)) :=
 (oo f (cx i)).

Remark Hc : (NT_Comm c).
Proof.
  Unfold c NT_Comm; Simpl; Intros.
  Rewrite <- oo_assoc.
  Rewrite ii_sx.
  Assert HH := (nt_comm cx).
  Simpl in HH.
  Rewrite HH.
  Rewrite ii_sx.
  Reflexivity.
Qed.

Definition cocone_comp : (Cocone y).
Proof.
  Split with c; Exact Hc.
Defined.

End Cocone_Comp.

Definition Colimit [u:A; cu:(Cocone u)] : Type :=
 (x:A)(Map_.inverse [f:(mor u x)](cocone_comp f cu)).

Definition colimit_mor [u:A; cu:(Cocone u); ccu:(Colimit cu); x:A; cx:(Cocone x)] : 
 (mor u x) := (Map_.inverse_map (ccu x) cx).

Lemma colimit_mor_eq_cocone : (u:A; cu:(Cocone u); ccu:(Colimit cu))
  (x:A; cx:(Cocone x))
  (cocone_comp (colimit_mor ccu cx) cu)==cx.
Proof.
  Unfold colimit_mor; Intros.  
  Rewrite (Map_.inverse_dx_eq (ccu x)).
  Reflexivity.
Qed.

Lemma colimit_mor_eq_mor : (u:A; cu:(Cocone u); ccu:(Colimit cu))
 (x:A; f:(mor 1!A u x))(ccu x (cocone_comp 2!x f cu))=f.
Proof.
  Unfold colimit_mor; Intros.  
  Rewrite (Map_.inverse_sx_eq (ccu x)).
  Reflexivity.
Qed.

Lemma colimit_mor_unique : (u:A; cu:(Cocone u); ccu:(Colimit cu))
  (x:A; f,g:(mor u x))
  (cocone_comp f cu)==(cocone_comp g cu) -> f==g.
Proof.
  Intros.
  (* !!!! injective_of_inverse in map_.v *)
  Assert HH := (Map_.cosection_inj (Map_.cosection_of_inverse (ccu x))).
  Apply HH.
  Assumption.
Qed.

Lemma colimit_mor_char : (u:A; cu:(Cocone u); ccu:(Colimit cu))
  (x:A; cx:(Cocone x); f:(mor u x))
  (cocone_comp f cu)==cx -> f==(colimit_mor ccu cx).
Proof.
  Intros.
  Apply (colimit_mor_unique ccu).
  Rewrite H.
  Rewrite colimit_mor_eq_cocone.
  Reflexivity.
Qed.

Remark colimit_mor_ii :
  (u:A; cu:(Cocone u); ccu:(Colimit cu))
  (v:A; cv:(Cocone v); ccv:(Colimit cv))
  (oo (colimit_mor ccv cu) (colimit_mor ccu cv))==(ii u).
Proof.
  Intros.
  Apply (colimit_mor_unique ccu).
  Apply Cocone_extens; Simpl; Intros.
  Rewrite ii_sx.
  Rewrite <- oo_assoc.
  Change
    (eq_mor
      ((cocone_comp (colimit_mor ccv cu)
                     (cocone_comp (colimit_mor ccu cv) cu)) i)
      (cu i)).
  Rewrite (colimit_mor_eq_cocone ccu cv).
  Rewrite (colimit_mor_eq_cocone ccv cu).
  Split.
Qed.

Definition colimit_unique :
  (u:A; cu:(Cocone u); ccu:(Colimit cu))
  (v:A; cv:(Cocone v); ccv:(Colimit cv))
  (Inverse_Mor (colimit_mor ccu cv)).
Proof.
  Intros; Split with (colimit_mor ccv cu).
  Hints Resolve colimit_mor_ii.
  Abstract Split; Auto.
Qed.

End Colimit.

Section Cocone_Pullback.

Variable A,I,J:Cat; F:(Functor I J).

Variable D:(Functor J A); x:A; cx:(Cocone D x).

(* !!!! nt_data *)
Local D' := (Functor_.morph_comp D F).

Local eta_data [i:I] : (mor (D' i) (constant_fct I x i)) :=
  (cx (F i)).    

Remark eta_comm : (NT_Comm eta_data).
Proof.
  Unfold eta_data NT_Comm; Simpl; Intros.
  Assert HH := (nt_comm cx).
  Simpl in HH.
  Rewrite HH.
  Repeat Rewrite ii_sx.
  Reflexivity.
Qed.

Definition cocone_pullback : (Cocone D' x).
Proof.
  Split with eta_data.
  Exact eta_comm.
Defined.

End Cocone_Pullback.

End Colimit2_.
