(* tag: Grothendieck pretopologies *)

(** * Grothendieck pretopologies *)

Require Export cat.
Require Export fibprod.
Require Export target.
Require Export topology.

Implicit Arguments On.

Module Pretopology.

Export Cat.
Export Fibprod.
Export Target.
Export Topology.

Section Pretopology.

Variable A:Cat; pb:(has_fibprod A).

Record Pretopology : Type := {
  pretopology: (u:A; a:(Target u))Type;
  pretopology_isom : (x,y:A; f:(mor x y); Hf:(Inv f))
   (pretopology y (target_of_mor f));
  pretopology_change :
   (u:A; a:(Target u); Ha:(pretopology u a); v:A; h:(mor v u))
   (pretopology v (pullback_target pb a h));
  pretopology_comp : (u:A; a:(Target u); Ha:(pretopology u a))
   (b:(x:A; f:(mor x u); Hf:(a x f))(Target x))
   (Hb:(x:A; f:(mor x u); Hf:(a x f))(pretopology x (b x f Hf)))
   (pretopology u (target_comp b))
 }.

(** ** Topology generated by a pretopology *)

Section Pretopology_Gen.

Variable K:Pretopology.

Record Pretopology_Gen_Data [u:A; a:(Sieve u)] : Type := {
  pretopology_gen_target :> (Target u);
  pretopology_gen_pretopology : (pretopology K pretopology_gen_target);
  pretopology_gen_sub : (target_sub pretopology_gen_target a)
}.

Definition pretopology_gen_maximal : (u:A; s:(Sieve u))
 (is_maximal_target s)->(Pretopology_Gen_Data s).
Proof.
  Unfold is_maximal_target; Intros.
  Split with (target_of_mor (ii u)).
  Apply pretopology_isom.
  Split with (ii u).
  Abstract Split; Unfold is_inv_sx is_inv_dx; Auto.
  Abstract Unfold target_sub; NewDestruct 1; Auto.  
Defined.

Definition pretopology_gen_larger : (u:A; s,t:(Sieve u))
 (Pretopology_Gen_Data s)->(target_sub s t)->(Pretopology_Gen_Data t).
Proof.
  Intros u s t Hs Hst.
  Split with (pretopology_gen_target Hs).
  Apply pretopology_gen_pretopology.
  Abstract NewDestruct Hs; Unfold target_sub; Auto.
Defined.

Remark rmk_pretopology_gen_transport : (u,v:A; h:(mor u v))
 (s:(Sieve v); Hs:(Pretopology_Gen_Data s))
 (target_sub (pullback_target pb Hs 5!u h) (sieve_transport 2!u h s)).
Proof.
  Intros.
  Unfold target_sub; Simpl.
  Unfold target_transport.
  Intros.
  NewDestruct Hf.
  LetTac p := (pb h (target_mor f)).
  Change (sieve s (oo h (fct (base v) (pullback p)))).
  Rewrite (pullback_eq p).
  Apply sieve_prop.
  Apply pretopology_gen_sub with Hs.
  Apply target_prop.
Qed.

Definition pretopology_gen_transport : (u,v:A; h:(mor u v))
 (s:(Sieve v); Hs:(Pretopology_Gen_Data s))
 (Pretopology_Gen_Data (sieve_transport h s)).
Proof.
  Intros.
  Split with (pullback_target pb Hs h).
  Apply pretopology_change.
  Apply pretopology_gen_pretopology.
  Apply rmk_pretopology_gen_transport.
Defined.

Definition target_of_sieve [
  u:A; s:(Sieve u); t:(Pretopology_Gen_Data s)
 ] : (Target u) :=
 (pretopology_gen_target t).

Lemma target_of_sieve_sub [
  u:A; s:(Sieve u); t:(Pretopology_Gen_Data s)
 ] : (target_sub (target_of_sieve t) s).
Proof.
  Simpl; Intros.
  Apply pretopology_gen_sub.
Qed.

Definition target_of_sieve_prop [
  u:A; s:(Sieve u); t:(Pretopology_Gen_Data s)
 ] : (pretopology K (target_of_sieve t)) :=
 (pretopology_gen_pretopology t).

Definition pretopology_gen_transitivity : (u:A; s,t:(Sieve u))
 (Hs:(Pretopology_Gen_Data s))
 (H:(x:A; f:(mor x u); Hf:(sieve s f))
  (Pretopology_Gen_Data (sieve_transport f t)))
 (Pretopology_Gen_Data t).
Proof.
  (* !!!! Attenzione muove in identificatori opachi! *)
  Intros.
  Split with
    (target_comp
       [x:A; f:(mor x u); Hf:(pretopology_gen_target Hs f)]
       (H x f (pretopology_gen_sub Hf))).
  Apply pretopology_comp.
  Apply pretopology_gen_pretopology.
  Intros.
  Apply pretopology_gen_pretopology.
  Unfold target_sub; Intros.
  NewDestruct Hf.
  Exact (pretopology_gen_sub Hg).
Defined.

(* !!!! *)
(*
Section Pretopology_Gen_Comp.

Variable u:A; s:(Sieve u); Hs:(Pretopology_Gen_Data s).
Variable t:(x:A; f:(mor x u); Hf:(sieve s f))(Sieve x).
Variable Ht:(x:A; f:(mor x u); Hf:(sieve s f))(Pretopology_Gen_Data (t Hf)).

Local a : (Target u) := (target_of_sieve Hs).

Local r [x:A; f:(mor x u); Hf:(a f)] : (Target x) :=
 (target_of_sieve (Ht (target_of_sieve_sub Hf))).

Local p : (Target u) := (target_comp r).

Lemma p_sub : (target_sub p (sieve_comp t)).
Proof.
  Unfold p r a target_sub; Intros; Simpl.
  NewDestruct Hf.
  Split with (target_of_sieve_sub Hf).
  Apply (target_of_sieve_sub Hg).
Qed.

Definition pretopology_gen_comp : (Pretopology_Gen_Data (sieve_comp t)).
Proof.
  Split with p; Unfold p.
  Apply pretopology_comp.
  Unfold a.
  Apply target_of_sieve_prop.
  Intros.
  Unfold r.
  Apply target_of_sieve_prop.
  Apply p_sub.
Defined.

End Pretopology_Gen_Comp.
*)

Definition Pretopology_Gen : (Topology A).
Proof.
  Split with Pretopology_Gen_Data; Intro; Intros.
  Apply pretopology_gen_maximal; Auto.
  Apply pretopology_gen_transport; Assumption.
  Apply pretopology_gen_transitivity with s; Assumption.
Defined.

End Pretopology_Gen.

End Pretopology.

End Pretopology.
