(* tag: Limits in the category of sets - new version *)

(** * Limits in the category of sets *)

(** We prove that the category of sets has limits, that is any diagram
in [Small_Set] has a limit. *)

Implicit Arguments On.

Require Export colimit.
Require Export small_set.
Require quot.
Require map.

Module Set_Colimit2.

Export Small_Set.
Export Colimit2.
Import Quot.
Import Map.

Section Set_Colimit.

Variable I:Cat; D:(Functor I Small_Set).

Inductive E : Type :=
 | Eintro: (i:I; elt:(D i)) E.

Definition Eindex [x:E] : I :=
 Cases x of (Eintro i elt) => i end.

Definition Ecarrier [x:E] : Small_Set :=
 (D (Eindex x)).
(* !!!!
 Cases x of (Eintro i elt) => (D i) end.
*)

Definition Eelt [x:E] : (Ecarrier x) :=
 <[x:E](Ecarrier x)>Cases x of (Eintro i elt) => elt end.

(** !!!! Note: see the old set_colimit.v *)

(** !!!! We introduce an equivalence relation [Q] on [E] as follows.  We
define [P] such that [(P (f x) x)] for each [x:E] and each arrow [f] of
[D], whenever the application [(f x)] make sense.  Then [Q] is defined
as the equivalence relaton generated by [P].  *)

Inductive P : (x,y:E) Prop :=
 Pintro : (i,j:I; f:(mor i j); x:(D i); y:(D j))
   (fct D f x)==y -> (P (Eintro x) (Eintro y)).

Definition Q : (Equivalence E) := (eqgen P).

(* !!!! cambiare nome! *)
Definition u : Small_Set := (quot Q).

(** We want to prove that [u] is the colimit of [D], that is, [U] is the
vertex of a cocone over [D] which has the universal property.  Let start
by build the cocone.  *)

Local eta_data [i:I] : (mor (D i) (constant_fct I u i)) :=
  [x:(D i)](class Q (Eintro x)).

Remark eta_comm : (NT_Comm eta_data).
Proof.
  Unfold NT_Comm; Simpl; Intros.
  Apply nondep_extensionality.
  Intros e. 
  Unfold eta_data; Simpl.
  Apply class_eq with Q:=Q.
  Unfold Q.
  Apply eqv_sym.
  Apply eqgen_intro.
  EApply Pintro; Simpl.
  Reflexivity.
Qed.

Definition set_colimit_cocone : (Cocone D u).
Proof.
  Split with eta_data; Exact eta_comm.
Defined.

(** We prove that the cocone [cu] has the suitable universal property.
For every cocone [cv] there is a morphism [s] from [u] to [v].  This
morphism [s] will be the data required to build the colimit. *)

Section Universal_Map.

Variable v:Small_Set; cv:(Cocone D v).

Definition h [x:E] : v :=
  (cv (Eindex x) (Eelt x)).

(* !!!!
Definition h [x:E] : v :=
 Cases x of (Eintro i elt) => (cocone cv i elt) end.
*)

Remark h_compat : (Compatible Q h).
Proof.
  Unfold Q.
  Apply eqgen_compat.
  NewDestruct 1.
  Unfold h.
  Rewrite <- H.
  Change (cv i x)=(oo (cv j) (fct D f) x).
  Replace (oo (cv j) (fct D f)) with (cv i).
  Reflexivity.
  Rewrite nt_comm.
  Simpl.
  Apply nondep_extensionality.
  Intro a.
  Reflexivity.
Qed.

Definition s : u->v.
Proof.
  Apply fact with Q:=Q f:=h.
  Apply h_compat.
Defined.

End Universal_Map.

(** The following remark is the proof needed for proving
[set_colimit_theory] below. *)
Remark rmk1 : (w:Small_Set)
  (* !!! cocone comp non va bene cosi'. *)
 (Inverse_Theory [f:(u ->w)]
   (cocone_comp 5!w f set_colimit_cocone) (s 1!w)).
Proof.
  Unfold s; Split; Simpl.
  Intro f.
  Apply nondep_extensionality.
  Intro a; Simpl.
  NewDestruct (class_surj a).
  NewDestruct H.
  Rewrite fact_eq.
  NewDestruct x; Trivial.
  Unfold Cocone cocone_comp; Intros cw.
  Apply nt_extens; Simpl; Intros i.
  Apply nondep_extensionality.
  Intros x.
  Unfold eta_data.
  Rewrite fact_eq.
  Reflexivity.
Qed.

Definition set_colimit : (Colimit set_colimit_cocone).
Proof.
  Unfold Colimit; Simpl.
  Intro w.
  Split with (!s w).
  Apply rmk1.
Defined.

End Set_Colimit.

End Set_Colimit2.
