(* tag: support of a list *)

(** * Support of a list *)

Require Export list_.

Module Support_.

Export List_.

Section Support.

Variable A:Type.

Inductive Support : (l:(List A)) Type :=
 | support_intro_head : (x:A; xs:(List A))
     (l:(List A); Hl:l==(cons x xs)) (Support l)
 | support_intro_tail : (x:A; xs:(List A); a:(Support xs))
     (l:(List A); Hl:l==(cons x xs)) (Support l).

Definition support_head [x:A; xs:(List A)] : (Support (cons x xs)).
Proof.
  Intros.
  Apply support_intro_head with x xs.
  Reflexivity.
Defined.

Definition support_tail [x:A; xs:(List A); a:(Support xs)] :
 (Support (cons x xs)).
Proof.
  Intros.
  Apply support_intro_tail with x xs.
  Exact a.
  Reflexivity.
Defined.

Definition support_elt [l:(List A); a:(Support l)] : A.
Proof.
  NewInduction 1.
  Exact x.
  Exact IHa.
Defined.

Definition support_list [l:(List A); a:(Support l)] : (List A).
Proof.
  NewInduction 1.
  Exact (cons x xs).
  Exact IHa.
Defined.

End Support.

End Support_.
